import json
from skimage import io #Загрузка и сохранение картинок
from skimage.transform import downscale_local_mean #Даунскэйл изображений
from skimage.filters import meijering #Фильтр
from skimage.exposure import equalize_adapthist
from skimage.color import rgb2gray
import numpy as np
import time

def image_former(picture, crop:int=4):    
    picture = rgb2gray(picture)
    #Кропаю изображение
    if crop > 1:
        picture = downscale_local_mean(picture,(crop,crop))
    #Нормализую яркость
    picture = equalize_adapthist(picture,clip_limit=0.015,nbins=4096)
    #Накладываю фильтр для уменьшения избыточности
    picture = meijering(picture,sigmas=(1,2),mode='reflect') # type: ignore
    image = np.fft.fft2(picture)
    return image

def shift_finder(reference, image, crop:int = 4):
    corr_m = np.fft.ifftshift(np.fft.irfft2(reference*np.conjugate(image)))
    dy, dx = np.unravel_index (corr_m.argmax(),corr_m.shape)
    y = (corr_m.shape[0]/2 - dy) * crop
    x = (corr_m.shape[1]/2 - dx) * crop
    return([x,y])

def new_reference (picture, id, pos, crop):
    #Сохраняю референстное изображения для возможности просмотра
    snap_time = int(time.time())
    snap_name = f"./refs/{snap_time}.jpg"
    io.imsave(snap_name,picture)
    # Формирую образ изображения и сохраняю его для последующего использования
    image = image_former(picture,crop)
    img_name = f"./refs/{snap_time}"
    np.save(img_name,image,allow_pickle=True)
    #Формирую словарь с базовой информацией о пресете
    # pos = [0,0,0] #Заглушка
    x,y,z = pos
    references = []
    for i in range(0,11):
        r = {"Ref": str(i), "Image":f"{snap_time}.npy", "x":"0", "y":"0"}
        references.append(r)
    preset_info = {"ID":str(id), "Picture":snap_time, "Position": {"Pan":x, "Tilt": y, "Focal":z}, "References":references}
    f = open(f"./presets/{str(id)}.json","w")
    json.dump(preset_info,f,indent=4)
    f.close()
    return snap_time
    
def shift_estimate (picture, id,crop):
    #Загружаю референсы
    f = open(f"presets/{str(id)}.json","r")
    data = json.load(f)
    f.close()
    snaps_list = data["References"]
    #Загружаю референсы и смещения
    references = []
    refs_names = []
    xy = []
    
    for i in snaps_list:
        image_name = i["Image"]
        sh_x = i["x"]
        sh_y = i["y"]
        snap_exist = False
        for j in refs_names:
            if image_name == j:
                snap_exist = True
        if snap_exist == False:
            refs_names.append(image_name)
            xy.append([sh_x,sh_y])
    for i in refs_names:
        ref = np.load(f"./refs/{i}",allow_pickle=True)
        references.append(ref)
        # print (ref.shape)
    # for i in data["References"]:
    #     image_id = i["Image"]
    #     x = i["x"]
    #     y = i["y"]
    #     ref = np.load(f"./refs/{image_id}",allow_pickle=True)
    #     references.append(ref)
    #     xy.append([x,y])
    #Получаю размер изображения
    h_size, w_size, _ = picture.shape
    #Получаю образ снапа
    image = image_former(picture, crop)
    #Получаю сдвиг для каждого изображения
    snap_res = []
    for i, ref in enumerate(references):
        # print (ref.shape)
        ref_calc = shift_finder(ref,image, crop)
        #Здесь добавить вычитание сдвига из файла
        sx = xy[i][0]
        sy = xy[i][1]
        ref_calc[0] = ref_calc[0]+float(sx)
        ref_calc[1] = ref_calc[1]+float(sy)
        #Определяю длину вектора сдвига
        ref_calc.append(np.linalg.norm(ref_calc[:2]))
        # ref_calc.append(ref_calc[0]+float(sx))
        # ref_calc.append(ref_calc[1]+float(sy))
        snap_res.append(ref_calc)
    # print(snap_res)
       
    #Выкалываю точки снижающие девиацию
    if len(snap_res) > 2:
        while len(snap_res) > 2:
            dev = []
            for i in range (len(snap_res)):
                snap_poke = snap_res[:i] + snap_res[(i+1):]
                norm_poke = np.array(snap_poke)[:,2]
                deviation = np.std(np.array(norm_poke))
                dev.append(deviation)
            pop_index = np.argmin(dev)   
            snap_res.pop(pop_index) 
            
    # Выделяю результат
    snap_arr = np.array(snap_res)
    # Если сдвиг каждого кадра меньше половины кадра
    # print(len(snap_arr))
    if (np.max(np.abs(snap_arr[:,0])) > w_size / 3) or (np.max(np.abs(snap_arr[:,1])) > h_size / 3):
        x = int(round(snap_arr[0,0],0))
        y = int(round(snap_arr[0,1],0))
        # d = np.linalg.norm([np.std(snap_arr[:,0]),np.std(snap_arr[:,1])])
        d = float(1000)
    else:
        # print(snap_arr[:,0], snap_arr[:,1])
        x = int(round(np.mean(snap_arr[:,0]),0))
        y = int(round(np.mean(snap_arr[:,1]),0))
        # d = np.std(np.linalg.norm([snap_arr[:,0],snap_arr[:,1]]))
        d = np.linalg.norm([np.std(snap_arr[:,0]),np.std(snap_arr[:,1])])
        # print(snap_arr)
    #Надо добавить айди изображения в проде чтобы можно было понять относительно каких изображений считался сдвиг
    return(x,y,d,image)

def update_references(image, snapid, ref_list):
    #Получаю флаги на обновление часов и минут
    hr_flag = pupd_hr(int(snapid))
    min_flag = pupd_15(int(snapid))
    # hr_flag = 0
    startprs = ref_list["0"]
    # min_flag = True
    if hr_flag > 0:
        #Определяю timestamp снапа из пресета
        preset_ts = int(ref_list[str(hr_flag)])
        #Проверяю, есть ли снапу сутки
        day_dtime = 24*60*60
        if (int(snapid) - preset_ts >= day_dtime) or (int(preset_ts) == int(startprs)):
            #В таком случае сохраняю снап и на выводе даю id и имя снапа для дальнейшей записи
            filename = f"./refs/{str(snapid)}.npy"
            np.save(filename,np.array(image))
            return {str(hr_flag):filename.replace("./refs/","")}
        else: return None
    elif min_flag:
        #Если часовое обновление не нужно, то делаем 15 минутное
        filename = f"./refs/{str(snapid)}.npy"
        np.save(filename,np.array(image))
        old = ref_list["10"]
        return {"10":filename.replace("./refs/",""), "9":f"{old}.npy"}
    else: return None  
        # pass

#Выбирает id пресета для обновления в соответствии с временем (каждые 3 часа)
#На выходе число от 1 до 8, либо 0 если ничего не нужно обновлять
def pupd_hr (timestamp):
    t = time.localtime(timestamp)
    t_h = t.tm_hour
    if t_h == 1: return 1
    elif t_h == 4: return 2
    elif t_h == 7: return 3
    elif t_h == 10: return 4
    elif t_h == 13: return 5
    elif t_h == 16: return 6
    elif t_h == 19: return 7
    elif t_h == 22: return 8
    else: return 0
#Если время делится на 15 (+5) минут, то обновляет флаг обновление пресета для id 10 (и сдвига старого изображения в id 9)
def pupd_15 (timestamp):
    t = time.localtime(timestamp)
    t_m = t.tm_min
    if t_m == 5: return True
    elif t_m == 20: return True
    elif t_m == 35: return True
    elif t_m == 50: return True
    else: return False
    
    
def original_shift(image, presets, crop):
    ref_id = presets["0"]
    ref = np.load(f"refs/{ref_id}.npy",allow_pickle=True)
    shift = shift_finder(ref,image,crop)
    return shift