import os
import json
import cs_io
import camera #ONVIF

from fastapi import FastAPI, File, HTTPException
from fastapi.responses import FileResponse
from fastapi.responses import *


import requests

import shift
import uvicorn

import time
from skimage import io 
import numpy as np
import time
""" Локальные библиотеки """
from sio import *
from camera import *
from image import *

#image.py - формирование и загрузка изображений, поиск шифта
#camera.py - управление камерой
#manager.py - управление пресетами




#Проверяю существование папок
dirnames = next(os.walk("./"), (None, None, []))[1]  # [] if no file
if "refs" not in  dirnames:
    os.mkdir("./refs", mode=0o777)
if "snaps" not in  dirnames:
    os.mkdir("./snaps", mode=0o777)
if "models" not in  dirnames:
    os.mkdir("./models", mode=0o777)
if "presets" not in  dirnames:
    os.mkdir("./presets", mode=0o777)


#####При загрузке
#!/usr/bin/env python

#Загружаю конфиг из env
sh_ip = os.getenv("shifting_ip")
sh_port = os.getenv('shifting_port','80')
sh_user = os.getenv('shifting_user')
sh_password = os.getenv('shifting_password')
sh_model = os.getenv('shifting_model','DH-SD49225XA-HNR-S3')
sh_crop = os.getenv('shifting_crop','4')
sh_atype = os.getenv('shifting_atype','digest')
sh_nsnaps = os.getenv('shifting_nsnaps','100')
# Если есть ip,login,password - загружаю конфиг из переменной окружения
if sh_ip is not None and sh_user is not None and sh_password is not None:
    camera_config=cs_io.load_config_env(sh_ip,sh_port,sh_user,sh_password,sh_model,sh_crop,sh_atype,sh_nsnaps)
    print("Config loaded from environments")
else:
    camera_config = cs_io.load_config()
    print("Config loaded from config file")
aov_profile = []
profilename = camera_config["model"]["focal"]["profile"]
if os.path.isfile(f"models/{profilename}"):
    aov_profile = np.load(f"models/{profilename}")


#####Подключение к камере
cam = camera.connection(camera_config)
while cam is None:
    print("Camera connection error. Reconnecing after 30 s...")
    time.sleep(30)
    cam = camera.connection(camera_config)
print ("Camera is connected. Getting parametrs...")
connected  = False
med = camera.get_media(cam)
while med is None:
    print("Media profile load error. Reconnecting after 30 s...")
    med = camera.get_media(cam)
    time.sleep(30)
pt = camera.get_ptz(cam)
while med is None:
    print("PTZ profile load error. Reconnecting after 30 s...")
    pt = camera.get_ptz(cam)
    time.sleep(30)
media, mediat = med
ptz,ptzt = pt
print("Ok")

app = FastAPI(
    title = "Camera shifting correction API",
    version = 0.3,
    )

##### Коррекция
@app.get("/correct/{preset_id}")
async def camera_correction (preset_id: int):
    calc_time = []
    id = preset_id
    ### Этап 1 - поиск сдвига и обновление пресетов
    #Проверим, существует ли пресет
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Удаляю устаревшие снапы
    cs_io.snap_remover(int(camera_config["nsnap"]))
    #Получаю координаты в которые нужно сдвинуть камеру
    position = data["Position"]
    x,y,z = [position["Pan"], position["Tilt"], position["Focal"]]
    #Конвертирую координаты из градусов в onvif
    x = deg2pan (x, camera_config)
    y = deg2tilt(y, camera_config)
    z = mm2zoom (z,camera_config)
    start_time = time.clock_gettime(time.CLOCK_MONOTONIC)
    # Перехожу в координаты пресета
    goto_status = camera.goto_ptz(x,y,z,ptz,mediat)
    if goto_status is False:
        raise HTTPException(status_code=400, detail="Camera connection error")
    #Ожидание репозиционирования
    status = ptz_status(ptz, mediat)
    while status is False:
        status = ptz_status(ptz, mediat)
    time.sleep(0.7)
    #Получаю снапшот
    snap_img = get_snap(media,mediat,camera_config)
    if snap_img is None:
        raise HTTPException(status_code=400, detail="Camera connection error")
    # old_snap_id = int(time.time())
    # io.imsave(f"./snaps/{str(snap_id)}.jpg",snap_img)
    #Получаю сдвиг
    cstart_time = time.clock_gettime(time.CLOCK_MONOTONIC)
    dx, dy, dev, image = shift_estimate(picture=snap_img,id=id,crop=int(camera_config["crop"]))
    # print(dx,dy,dev)
    cend_time = time.clock_gettime(time.CLOCK_MONOTONIC)
    calc_time.append(cend_time-cstart_time)
    #Сохраняю снапшот
    snap_id = int(time.time())
    snap_id_uc = snap_id
    io.imsave(f"./snaps/{str(snap_id)}.jpg",snap_img)
    #Сравниваю сдвиг и девиацию с порогами. Если ниже порога, то готовлюсь к обновлению
    deviationpr_th = float(camera_config["model"]["thresholds"]["deviationpr"])
    h_th = float(camera_config["model"]["resolution"]["heigth"]) /3
    w_th = float(camera_config["model"]["resolution"]["width"]) /3
    presets = {}
    for i in data["References"]:
        d_index = i["Ref"]
        d_data = i["Image"]
        d_data = d_data.replace(".npy","")
        presets[d_index]=d_data
        old_name = ""
    if deviationpr_th >= dev:
        if (dx < w_th) and (dy < h_th):
            ref_u = update_references(image,snap_id,presets)
            sorigx, sorigy = original_shift(image,presets,crop=int(camera_config["crop"]))
            preset_th = float(camera_config["model"]["thresholds"]["presetsth"])
            if (abs(sorigx - dx) > preset_th) or (abs(sorigy - dy) > preset_th):
                ref_u = None
            #Проверяю, что есть что обновлять
            if ref_u is not None:
                #Проверяю часовое обновление или 15 минутное (10 пресет)
                keys = list(ref_u.keys())
                hour = True
                for i in keys:
                    if int(i) == 10:
                        hour = False
                #Если по часам, то обновляем конкретный пресет
                #Если 15 минутное, то переносим из 10 в 9, а 10 обновляем
                if hour>0:
                    key = keys[0]
                    #Ищу в data пресет с нужным id 
                    for i in range(len(data["References"])):
                        if str(key) == data["References"][i]["Ref"]:
                            old_name = data["References"][i]["Image"]
                            data["References"][i]["Image"] = ref_u[key]
                            data["References"][i]["x"] = str(dx)
                            data["References"][i]["y"] = str(dy)
                else:
                    for i in range(len(data["References"])):
                        if "10" == data["References"][i]["Ref"]:
                            for j in range(len(data["References"])):
                                if "9" == data["References"][j]["Ref"]:
                                    old_name = data["References"][j]["Image"]
                                    data["References"][j]["Image"] = data["References"][i]["Image"]
                                    data["References"][j]["x"] = data["References"][i]["x"]
                                    data["References"][j]["y"] = data["References"][i]["y"]
                            data["References"][i]["Image"] = ref_u["10"]
                            data["References"][i]["x"] = str(dx)
                            data["References"][i]["y"] = str(dy)
    #Перезаписываю конфиг
    try:
        f = open(f"presets/{str(id)}.json","w")
        json.dump(data,f,indent=4)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно записать пресет")
    #Удаляю устаревшие референсы
    cs_io.clear_old_refs() 
    ### Этап 2 - Коррекция
    #Если девиация выше порога, то коррекция не осуществляется
    deviation_th = float(camera_config["model"]["thresholds"]["deviation"])
    if deviation_th >= dev:
        #Выполняется коррекция или выводится сообщение что коррекция не требуется
        correction_th = float(camera_config["model"]["thresholds"]["correction"])
        if (abs(dx) > correction_th) or (abs(dy) > correction_th):
            #Требуется коррекция
            #Получаю координаты
            pos = get_position(ptz,mediat)
            if pos is None:
                raise HTTPException(status_code=400, detail="Camera connection error")
            x,y,z = pos
            x = pan2deg(x,camera_config)
            y = tilt2deg(y,camera_config)
            z = zoom2mm(z,camera_config)
            #Нахожу углы обзора и сдвиг
            aov = []
            if len(aov_profile) == 0:
                aov = calc_aov((z,z),camera_config)
            else:
                aov = lin_aov(z,aov_profile)
            x_shd, y_shd = calc_degshift2(dx,dy, aov, camera_config) 
            #Выполняю поворот камеры
            inv_flag = float(camera_config["model"]["position"]["inversion"])
            if inv_flag == True:
                x = x - round(x_shd,1)
                y = y + round(y_shd,1)
            else:
                x = x + round(x_shd,1)
                y = y - round(y_shd,1)
            #Перехожу в координаты камеры
            x = deg2pan(x,camera_config)
            y = deg2tilt(y,camera_config)
            z = mm2zoom(z,camera_config)
            #Позиционирую
            goto_status = goto_ptz(x,y,z,ptz, mediat)
            if goto_status is False:
                raise HTTPException(status_code=400, detail="Camera connection error")
            #Ожидание репозиционирования
            status = ptz_status(ptz, mediat)
            while status is False:
                status = ptz_status(ptz, mediat)
            time.sleep(0.3)
            # status = False
            # while status == False:
            #     status = ptz_status(ptz, mediat)
            #Получаю снап с камеры и сохраняю
            del snap_img
            snap_img = get_snap(media,mediat,camera_config)
            if snap_img is None:
                raise HTTPException(status_code=400, detail="Camera connection error")
            snap_id = int(time.time())
            io.imsave(f"./snaps/{str(snap_id)}.jpg",snap_img)
            #Делаю повторную оценку
            cstart_time = time.clock_gettime(time.CLOCK_MONOTONIC)
            dxc, dyc, devc, _ = shift_estimate(picture=snap_img,id=id,crop=int(camera_config["crop"]))
            cend_time = time.clock_gettime(time.CLOCK_MONOTONIC)
            calc_time.append(cend_time-cstart_time)
            
            end_time = time.clock_gettime(time.CLOCK_MONOTONIC)
            #Коррекция не требуется
            return (
                {"Status": "Corrected",
                "Shifting": [dxc, dyc, devc],
                "Snap": snap_id,
                "Snap_uncr": snap_id_uc,
                "time":
                    {
                        "total": end_time - start_time,
                        "calculation": calc_time
                    },
                "WithoutCorrection": [dx, dy,dev]
                    
                }
            )       
            

        else:
            end_time = time.clock_gettime(time.CLOCK_MONOTONIC)
            #Коррекция не требуется
            return (
                {"Status": "No need",
                "Shifting": [dx, dy, dev],
                "Snap": snap_id,
                "time":
                    {
                        "total": end_time - start_time,
                        "calculation": calc_time
                    }
                    
                }
            )            
    else:
        end_time = time.clock_gettime(time.CLOCK_MONOTONIC)
        #Ошибка оценки сдвига
        return (
            {"Status": "Calculation error",
             "Shifting": [dx, dy, dev],
             "Snap": snap_id,
             "time":
                {
                    "total": end_time - start_time,
                    "calculation": calc_time
                }
                 
            }
        )



#####Рассчет сдвига
@app.get("/sc/{preset_id}")
async def shifting_calcularion (preset_id:int):
    id = preset_id
    #Проверим, существует ли пресет
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Удаляю устаревшие снапы
    cs_io.snap_remover(int(camera_config["nsnap"]))
    #Получаю координаты в которые нужно сдвинуть камеру
    position = data["Position"]
    x,y,z = [position["Pan"], position["Tilt"], position["Focal"]]
    #Конвертирую координаты из градусов в onvif
    x = deg2pan (x, camera_config)
    y = deg2tilt(y, camera_config)
    z = mm2zoom (z,camera_config)
    # Перехожу в координаты пресета
    goto_status = camera.goto_ptz(x,y,z,ptz,mediat)
    if goto_status is False:
        raise HTTPException(status_code=400, detail="Camera connection error")
    #Ожидание репозиционирования
    status = ptz_status(ptz, mediat)
    while status is False:
        status = ptz_status(ptz, mediat)
    time.sleep(0.5)
    #Вывожу новые координаты камеры в градусах
    pos = get_position(ptz,mediat)
    if pos is None:
        raise HTTPException(status_code=400, detail="Camera connection error")
    xr,yr,zr = pos
    #Получаю снапшот
    snap_img = get_snap(media,mediat,camera_config)
    if snap_img is None:
        raise HTTPException(status_code=400, detail="Camera connection error")
    #Получаю сдвиг
    dx, dy, dev, image = shift_estimate(picture=snap_img,id=id,crop=int(camera_config["crop"]))
    #Сохраняю снапшот
    snap_id = int(time.time())
    io.imsave(f"./snaps/{str(snap_id)}.jpg",snap_img)
    #Сравниваю сдвиг и девиацию с порогами. Если ниже порога, то готовлюсь к обновлению
    deviation_th = float(camera_config["model"]["thresholds"]["deviation"])
    h_th = float(camera_config["model"]["resolution"]["heigth"]) /3
    w_th = float(camera_config["model"]["resolution"]["width"]) /3
    presets = {}
    for i in data["References"]:
        d_index = i["Ref"]
        d_data = i["Image"]
        d_data = d_data.replace(".npy","")
        presets[d_index]=d_data
        old_name = ""
    if deviation_th >= dev:
        if (dx < w_th) and (dy < h_th):
            ref_u = update_references(image,snap_id,presets)
            sorigx, sorigy = original_shift(image,presets,crop=int(camera_config["crop"]))
            preset_th = float(camera_config["model"]["thresholds"]["presetsth"])
            if (abs(sorigx - dx) > preset_th) or (abs(sorigy - dy) > preset_th):
                ref_u = None
            #Проверяю, что есть что обновлять
            if ref_u is not None:
                #Проверяю часовое обновление или 15 минутное (10 пресет)
                keys = list(ref_u.keys())
                hour = True
                for i in keys:
                    if int(i) == 10:
                        hour = False
                #Если по часам, то обновляем конкретный пресет
                #Если 15 минутное, то переносим из 10 в 9, а 10 обновляем
                if hour:
                    key = keys[0]
                    #Ищу в data пресет с нужным id 
                    for i in range(len(data["References"])):
                        if str(key) == data["References"][i]["Ref"]:
                            old_name = data["References"][i]["Image"]
                            data["References"][i]["Image"] = ref_u[key]
                            data["References"][i]["x"] = str(dx)
                            data["References"][i]["y"] = str(dy)
                else:
                    for i in range(len(data["References"])):
                        if "10" == data["References"][i]["Ref"]:
                            for j in range(len(data["References"])):
                                if "9" == data["References"][j]["Ref"]:
                                    old_name = data["References"][j]["Image"]
                                    data["References"][j]["Image"] = data["References"][i]["Image"]
                                    data["References"][j]["x"] = data["References"][i]["x"]
                                    data["References"][j]["y"] = data["References"][i]["y"]
                            data["References"][i]["Image"] = ref_u["10"]
                            data["References"][i]["x"] = str(dx)
                            data["References"][i]["y"] = str(dy)
    #Перезаписываю конфиг
    try:
        f = open(f"presets/{str(id)}.json","w")
        json.dump(data,f,indent=4)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно записать пресет")
    #Удаляю устаревшие референсы
    cs_io.clear_old_refs()
    
    return {"x_angle": dx, "y_angle": dy, "dev":dev, "snap_id": snap_id}
    
    
    #Удаляю старые рефы
    
    
    
    # return dx,dy,dev, ref_u
    
# ***************************************************************************************************
# Управление пресетами
# ***************************************************************************************************
##### Создание пресета
@app.put("/prs_add/{preset_id}" ,status_code=200)
async def create_preset (preset_id:int=0):
    # Определяю какой айди дать пресету
    if preset_id == 0:
        id = ffindID()
    else:
        id = preset_id
        if fprs_find(id):
            raise HTTPException(status_code=400, detail="Ошибка: Пресет уже существует")
    #Получаю координаты с камеры
    pos = get_position(ptz,mediat)
    if pos is None:
        raise HTTPException(status_code=400, detail="Camera connection error")
    x,y,z = pos
    #Конвертируем координаты в человекочитаемый вид
    x = pan2deg(x, camera_config)
    y = tilt2deg(y,camera_config)
    z = zoom2mm(z,camera_config)
    position = [x,y,z]
    # Загружаю с камеры снапшот
    snap_img = get_snap(media,mediat,camera_config)
    if snap_img is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить снапшот")
    snap_id = new_reference(snap_img,str(id),position, crop=int(camera_config["crop"]))
    return {"detail": "Success!","preset_id": id, "snap_id":str(snap_id), "position": {"pan": x, "tilt": y, "zoom": z}}

##### Реинициация опорных изображений, делая снапшот
@app.put("/prs_img_in", status_code = 200)
async def update_preset_img_in(id):
    #Проверяю существование пресета
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Получаю позицию из конфига
    position = data["Position"]
    pos = [position["Pan"], position["Tilt"], position["Focal"]]
    #Получаю новый снапшот
    snap_img = get_snap(media,mediat,camera_config)
    if snap_img is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить снапшот")
    #Формирую новый снап
    snap_id = new_reference(snap_img,str(id),pos, crop=int(camera_config["crop"]))
    #Удаляю старый снап
    old_snap_id = data["Picture"]
    if os.path.exists(f"./refs/{str(old_snap_id)}.jpg"):
        os.remove(f"./refs/{str(old_snap_id)}.jpg")
    if os.path.exists(f"./refs/{str(old_snap_id)}.npy"):
        os.remove(f"./refs/{str(old_snap_id)}.npy")
    references = data["References"]
    for i in references:
        image_name = i["Image"]
        if os.path.exists(f"./refs/{image_name}"):
            os.remove(f"./refs/{image_name}")
    return {"detail": "Success!","preset_id": id, "snap_id":str(snap_id), "position": {"pan": pos[0], "tilt": pos[1], "zoom": pos[2]}}  

##### Реинициация опорных изображений, загружаемое изображение   
@app.put("/prs_img_ext", status_code = 200)
async def update_preset_img_ext(id, loaded_image: bytes = File(None)):
    #Проверяю существование пресета
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Получаю позицию из конфига
    position = data["Position"]
    pos = [position["Pan"], position["Tilt"], position["Focal"]]
    #Получаю новый снапшот
    snap_img = io.imread(loaded_image,plugin='imageio')
    if snap_img is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить снапшот")
    #Формирую новый снап
    snap_id = new_reference(snap_img,str(id),pos, crop=int(camera_config["crop"]))
    #Удаляю старый снап
    old_snap_id = data["Picture"]
    if os.path.exists(f"./refs/{str(old_snap_id)}.jpg"):
        os.remove(f"./refs/{str(old_snap_id)}.jpg")
    if os.path.exists(f"./refs/{str(old_snap_id)}.npy"):
        os.remove(f"./refs/{str(old_snap_id)}.npy")
    references = data["References"]
    for i in references:
        image_name = i["Image"]
        if os.path.exists(f"./refs/{image_name}"):
            os.remove(f"./refs/{image_name}")
    return {"detail": "Success!","preset_id": id, "snap_id":str(snap_id), "position": {"pan": pos[0], "tilt": pos[1], "zoom": pos[2]}}  
    
#####Обновление координат
@app.put("/prs_ptz_upd", status_code = 200)
async def update_preset_ptz(id):
    #Проверяю, существует ли пресет
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Получаю координаты с камеры
    pos = get_position(ptz,mediat)
    if pos is None:
        raise HTTPException(status_code=400, detail="Camera connection error")
    x,y,z = pos
    #Конвертируем координаты в человекочитаемый вид
    x = pan2deg(x, camera_config)
    y = tilt2deg(y,camera_config)
    z = zoom2mm(z,camera_config)
    #Обновляю информацию в json
    position = data["Position"]
    #Обновляю координаты
    position["Pan"] = x
    position["Tilt"] = y
    position["Focal"] = z
    data["Position"] = position
    #Сохраняю файл
    f = open(f"./presets/{str(id)}.json","w")
    json.dump(data,f,indent=4)
    f.close()
    return {"detail": "Success!","preset_id": id, "snap_id":str(data["Picture"]) , "position": {"pan": x, "tilt": y, "zoom": z}}

##### Удаление пресета
@app.delete("/prs_del", status_code = 200)
async def delete_preset_tag(id):
    #Проверяю, существует ли пресет
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Удаляю файл пресета
    if os.path.exists(f"./presets/{str(id)}.json"):
        os.remove(f"./presets/{str(id)}.json")
    #Удаляю изображение и образы
    old_snap_id = data["Picture"]    
    if os.path.exists(f"./refs/{str(old_snap_id)}.jpg"):
        os.remove(f"./refs/{str(old_snap_id)}.jpg")
    if os.path.exists(f"./refs/{str(old_snap_id)}.npy"):
        os.remove(f"./refs/{str(old_snap_id)}.npy")
    references = data["References"]
    for i in references:
        image_name = i["Image"]
        if os.path.exists(f"./refs/{image_name}"):
            os.remove(f"./refs/{image_name}")
    return {"detail": "Success!","preset_id": id}

##### Список существующих пресетов
@app.get("/prs_list", status_code = 200)
async def preset_list ():
    result = []
    filenames = next(os.walk("./presets/"), (None, None, []))[2]  # [] if no file
    for names in sorted(filenames):
        id = int(str.strip(names,".json"))
        result.append(id)
    return result

##### Перевести камеру в пресет
@app.put("/pts_goto", status_code = 200)
async def goto_preset(id):
    #Проверяю, существует ли пресет
    if not fprs_find(id):
        raise HTTPException(status_code=400, detail="Ошибка: Пресет не существует")
    #Загружаю пресет из файла
    try:
        f = open(f"presets/{str(id)}.json","r")
        data = json.load(f)
        f.close()
    except: 
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    if data is None:
        raise HTTPException(status_code=400, detail="Ошибка: Невозможно загрузить пресет")
    #Получаю позицию из конфига
    position = data["Position"]
    x,y,z = [position["Pan"], position["Tilt"], position["Focal"]]
    #Конвертирую координаты из градусов в onvif
    xp = deg2pan (x, camera_config)
    yp = deg2tilt(y, camera_config)
    zp = mm2zoom (z,camera_config)
    #Перехожу в координаты пресета
    goto_status = camera.goto_ptz(xp,yp,zp,ptz,mediat)
    if goto_status is False: 
        raise HTTPException(status_code=400, detail="Camera connection error")
    #Вывожу новые координаты камеры в градусах
    pos = get_position(ptz,mediat)
    if pos is None: 
        raise HTTPException(status_code=400, detail="Camera connection error")
    xr,yr,zr = pos
#     #Конвертирую в человекочитаемый вид (градусы, мм)
    xr = pan2deg(xr, camera_config)
    yr = tilt2deg(yr,camera_config)
    zr = zoom2mm(zr,camera_config)
    return {"detail": "Success!","ptz":{"pan":str(xr), "tilt": str(yr), "zoom":str(zr)}}

#********************************************************************************************
#Загрузка изображения с камеры
#********************************************************************************************
@app.get("/dnld", status_code = 200)
async def download_image(img_id):
    snap_path = "./snaps/" + str(img_id) + ".jpg"
    ref_path = "./refs/" + str(img_id) + ".jpg"
    if os.path.exists(snap_path):
        return FileResponse(path=snap_path,media_type="image/jpeg",filename=str(img_id) + ".jpg")
    elif os.path.exists(ref_path):
        return FileResponse(path=ref_path,media_type="image/jpeg",filename=str(img_id) + ".jpg")



\
@app.get("/getzoom")
async def getz():
    pos = get_position(ptz,mediat)
    if pos is not None:
        _,_,z = pos
        zoom = zoom2mm(z,camera_config)
        return zoom
    else: raise HTTPException(status_code=400, detail="Camera connection error")

@app.get("/setzoom")
async def setz(zoom):
    pos = get_position(ptz,mediat)
    if pos is not None:
        x,y,z = pos
        z = mm2zoom(float(zoom),camera_config)
        camera.goto_ptz(x,y,z,ptz,mediat)
        return zoom
    else: raise HTTPException(status_code=400, detail="Camera connection error")

# @app.get("/setzoom2")
# async def seta(zoom):
#     x,y,z = get_position(ptz,mediat)
#     z = zoom
#     camera.goto_ptz(x,y,z,ptz,mediat)
#     return zoom

@app.get("/ptz_move")
async def ptx_move(pan=0, tilt=0,zoom=0):
    x,y,z = pan, tilt, zoom 
    x = deg2pan(float(pan),camera_config)
    y = deg2tilt_rel(float(tilt),camera_config)
    z = zoom2onvif(float(zoom),camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_left")
async def prx_left(fine:bool = False):
    x = -2.5
    if fine:
        x = -0.1
    y = 0
    z = 0
    x = deg2pan(x,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_right")
async def prx_right(fine:bool = False):
    x = 2.5
    if fine:
        x = 0.1
    y = 0
    z = 0
    x = deg2pan(x,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_down")
async def prx_down(fine:bool = False):
    y = -2.5
    if fine:
        y = -0.1
    x = 0
    z = 0
    y = deg2tilt_rel(y,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_up")
async def prx_up(fine:bool = False):
    y = 2.5
    if fine:
        y = 0.1
    x = 0
    z = 0
    y = deg2tilt_rel(y,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_zoomp")
async def prx_zoomp(fine:bool = False):
    z = 1
    if fine:
        z = 0.1
    x = 0
    y = 0
    z = zoom2onvif(z,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}

@app.get("/ptz_zoomm")
async def prx_zoomm(fine:bool = False):
    z = -1
    if fine:
        z = -0.1
    x = 0
    y = 0
    z = zoom2onvif(z,camera_config)
    camera.move_ptz(x,y,z,ptz,mediat)
    return {"detail": "Success!"}


if __name__ == "__main__":
    uvicorn.run(app, host="0.0.0.0", port=4080, log_level="info")







