from cs_io import load_config
import camera #ONVIF
from camera import *
from image import image_former, shift_finder
import time
import numpy as np



#Вычисление углов обзора при помощи оценки сдвига и углов поворота
def findAOV(x,y,anglex,angley,config):
    w= float(config["model"]["resolution"]["width"])
    h= float(config["model"]["resolution"]["heigth"])
    #Нахожу углы сдвига
    if x != 0:
        onex = anglex / (2 * x)
        aovx = onex * w
    else:
        aovx = 0
    if y != 0:
        oney = angley / (2 * y)
        aovy = oney * h
    else:
        aovy = 0
    return aovx,aovy

#Нахожу фокусное расстояние для каждого aov
def findfocal(aov,config):
    w= 1 / float(config["model"]["resolution"]["width"])
    h= 1 / float(config["model"]["resolution"]["heigth"])
    mtrx_angle = np.arctan(w/h)
    inch = (2.54 * 2/3)
    sizei= 1 / float(config["model"]["resolution"]["matrix_size"])
    sizemm = inch * sizei * 10
    sizew = sizemm * np.cos(mtrx_angle)
    sizeh = sizemm * np.sin(mtrx_angle)
    
    aovx,aovy = aov
    aovx = np.deg2rad(aovx)/2
    aovy = np.deg2rad(aovy)/2
    vx = np.tan(aovx)
    vy = np.tan(aovy)
    zx = sizew/(2*vx)
    zy = sizeh/(2*vy)
    return zx, zy

#Определение углов обзора для заданого фокусного расстояния
def aov_estimate(focal:float,shift:int,conf,iter:int=5):
    camera, media_profile, media_token, ptz_profile, ptz_token, config = conf
    #Запрашиваю координаты и перевожу камеру в нужное фокусное
    pos = get_position(ptz_profile, media_token)
    z = mm2zoom(focal,config)
    x, y, _ = pos
    goto_ptz(x,y,z,ptz_profile, media_token)
    status = ptz_status(ptz_profile, media_token)
    while status is False:
        status = ptz_status(ptz_profile, media_token)
    time.sleep(0.5)
    
    #Снова запрашиваю координаты и преобразую их в градусы
    del x, y, z, pos
    pos = get_position(ptz_profile, media_token)
    x,y,z = pos
    x = pan2deg(x,config)
    y = tilt2deg(y,config)
    z = zoom2mm(z,config)
    x = x
    
    #Вычисляю рассчетные углы обзора и определяю сдвиг в градусах
    aov = calc_aov([z,z],config)
    shiftingangle = calc_degshift(shift,shift,aov,config)
    print(f"Focal_aov: {aov[0]}\u00B0 {aov[1]}\u00B0", end='; ')
    print(f"Angle: {shiftingangle[0]}\u00B0 {shiftingangle[1]}\u00B0")
    
    #Определяю координаты сдвинутые на shift
    x_plus  = x + shiftingangle[0]
    x_minus = x - shiftingangle[0]
    y_plus  = y + shiftingangle[1]
    y_minus = y - shiftingangle[1]
    
    x_plus  = deg2pan(x_plus,config)
    x_minus = deg2pan(x_minus,config)
    y_plus  = deg2tilt(y_plus,config)
    y_minus = deg2tilt(y_minus,config)
    
    #Создаю опорник
    ref_img = get_snap(media_profile,media_token,config)
    ref = image_former(ref_img,int(config["crop"]))
    

    
        #Для оси Y
    ys = []
    for iteration in range(iter):
        #Устанавливаю камеру в позицию 1
        goto_ptz(pos[0],y_plus,pos[2],ptz_profile,media_token)
        status = ptz_status(ptz_profile,media_token)
        while status is False:
            status = ptz_status(ptz_profile,media_token)
        #Получаю снап и преобразую его в образ (позиция 1)
        snap_img = get_snap(media_profile,media_token,config)
        image = image_former(snap_img,int(config["crop"]))
        _, dy = shift_finder(ref,image,int(config["crop"]))
        ys.append(abs(dy))
        #Устанавливаю камеру в позицию 2
        goto_ptz(pos[0],y_minus,pos[2],ptz_profile,media_token)
        status = ptz_status(ptz_profile,media_token)
        while status is False:
            status = ptz_status(ptz_profile,media_token)
        #Получаю снап и преобразую его в образ (позиция 2)
        snap_img = get_snap(media_profile,media_token,config)
        image = image_former(snap_img,int(config["crop"]))
        _, dy = shift_finder(ref,image,int(config["crop"]))
        ys.append(abs(dy))
    # print (ys)
    
    goto_ptz(pos[0],pos[1],pos[2],ptz_profile,media_token)
    status = ptz_status(ptz_profile,media_token)
    while status is False:
        status = ptz_status(ptz_profile,media_token)
    
    #Для оси X
    xs = []
    for iteration in range(iter):
        #Устанавливаю камеру в позицию 1
        goto_ptz(x_plus,pos[1],pos[2],ptz_profile,media_token)
        status = ptz_status(ptz_profile,media_token)
        while status is False:
            status = ptz_status(ptz_profile,media_token)
        #Получаю снап и преобразую его в образ (позиция 1)
        snap_img = get_snap(media_profile,media_token,config)
        image = image_former(snap_img,int(config["crop"]))
        dx, _ = shift_finder(ref,image,int(config["crop"]))
        xs.append(abs(dx))
        #Устанавливаю камеру в позицию 2
        goto_ptz(x_minus,pos[1],pos[2],ptz_profile,media_token)
        status = ptz_status(ptz_profile,media_token)
        while status is False:
            status = ptz_status(ptz_profile,media_token)
        #Получаю снап и преобразую его в образ (позиция 2)
        snap_img = get_snap(media_profile,media_token,config)
        image = image_former(snap_img,int(config["crop"]))
        dx, _ = shift_finder(ref,image,int(config["crop"]))
        xs.append(abs(dx))
    # print (xs)
    
    goto_ptz(pos[0],pos[1],pos[2],ptz_profile,media_token)
    status = ptz_status(ptz_profile,media_token)
    while status is False:
        status = ptz_status(ptz_profile,media_token)
        
    #Получаю усредненный сдвиг по осям
    mean = np.mean(xs), np.mean(ys)
    print(f"Shifting: {mean[0]} {mean[1]}",end=' ')
    
    #Определяю реальные углы обзора
    taov = findAOV(mean[0],mean[1],shiftingangle[0],shiftingangle[1],config=config)
    if taov[0] == 0:
        taov = aov[0], taov[1]
    if taov[1] == 0:
        taov = taov[0],aov[1]
    print(f"Estimated AOV: {taov[0]} {taov[1]}")
    return taov


if __name__ == '__main__':
    n = 5 #Количество иттераций
    sh = 150 #Целевой сдвиг в пикселах
    #####Загрузка конфига и убираем кроп (для вычисления на ПК, большая точность)
    camera_config = load_config()
    camera_config['crop']=1
    #Определяем кратность зума камеры
    max_zoom = float(camera_config['model']['focal']['max'])
    min_zoom = float(camera_config['model']['focal']['min'])
    scale = int(max_zoom/min_zoom)
    zoom = []
    for i in range(1,scale+1):
        zoom.append(round(min_zoom*i,1))
    #Подключаююсь к камере
    cam = camera.connection(camera_config)
    while cam is None:
        print("Camera connection error. Reconnecing after 30 s...")
        time.sleep(30)
        cam = camera.connection(camera_config)
    print ("Camera is connected. Getting parametrs...")
    connected  = False
    med = camera.get_media(cam)
    while med is None:
        print("Media profile load error. Reconnecting after 30 s...")
        med = camera.get_media(cam)
        time.sleep(30)
    pt = camera.get_ptz(cam)
    while med is None:
        print("PTZ profile load error. Reconnecting after 30 s...")
        pt = camera.get_ptz(cam)
        time.sleep(30)
    media, mediat = med
    ptz,ptzt = pt
    print("Ok")
    
    conf = cam, media, mediat, ptz, ptzt, camera_config
    #Формирую вектора с AOV по каждой из осей
    aovx = []
    aovy = []
    for i in range(scale):
        print (f"zoom: {zoom[i]}")
        ax , ay = aov_estimate(zoom[i],sh,conf=conf,iter=n)
        aovx.append((ax))
        aovy.append((ay))
    arr= [zoom,aovx,aovy]
    arr = np.transpose(arr)
    timestamp = int(time.time())
    np.save(f"models/{str(timestamp)}.prifile",arr)