import numpy as np
from skimage.filters import meijering
from skimage.transform import downscale_local_mean, SimilarityTransform, warp
import multiprocessing.dummy as mp
import os

from skimage import io
from skimage.color import rgb2gray

#Функция формирует пару образов используя многопоточность
def image_former(image, crop_factor: int = 1):
    #Преобразуем изображение в ЧБ
    image = rgb2gray(image)
    # image = cv2.cvtColor(image,cv2.COLOR_RGB2GRAY)
    #Производим ресайз изображения в соответствии с кроп-фактором
    if crop_factor > 1:
        image = downscale_local_mean(image, (crop_factor,crop_factor))
    # Применяем фильтр и преобразование в частотную область
    # Если процессоров два или более - многопоточность.
    if os.cpu_count() < 2:
        default = meij(image,False)
        alternative = meij(image,True)
    else:
        with mp.Pool(2) as p:
            default, alternative = p.starmap(meij,[(image, False, crop_factor), (image, True, crop_factor)])
    return default, alternative

def shifting_calculation(img, ref, crop_factor):
    img1, img2 = img
    ref1, ref2 = ref
    c1 = np.fft.ifftshift(np.fft.irfft2(ref1*np.conjugate(img1)))
    c2 = np.fft.ifftshift(np.fft.irfft2(ref2*np.conjugate(img2)))
    dy1, dx1 = np.unravel_index (c1.argmax(),c1.shape)
    dy2, dx2 = np.unravel_index (c2.argmax(),c2.shape)
    y1 = (c1.shape[0]/2 - dy1) * crop_factor
    y2 = (c2.shape[0]/2 - dy2) * crop_factor
    x1 = (c1.shape[1]/2 - dx1) * crop_factor
    x2 = (c2.shape[1]/2 - dx2) * crop_factor  
    xe = abs(x1-x2)
    ye = abs(y1-y2)
    err = np.sqrt(np.square(xe)+np.square(ye))
    # print(err)
    return x1, y1, err


async def correlation(ref,img):
    return np.fft.ifftshift(np.fft.irfft2(ref*np.conjugate(img)))

#Подготовка образа изображения
def meij (image, alt, crop_factor:int = 1):
    #Применяем фильтры к изображению
    if not alt:
        result = meijering(image,sigmas=(1,3,5,9),mode='reflect')
    else:
        result = meijering(image,sigmas=(7,11,17,21),mode='reflect')
    return np.fft.rfft2(result)

def complex_sh(preset_str, image, crop):
    # preset_json = json.loads(preset_str)
    preset_json = preset_str
    # print("test")
    ids = []
    ids.append(preset_json["images"]["base"]["snap"])
    ids.append(preset_json["images"]["day"]["snap"])
    ids.append(preset_json["images"]["night"]["snap"])
    ids.append(preset_json["images"]["adaptive"]["snap"])
    #Отдаю функции, которая для каждого id вернет оценку сдвига
    res = []
    for refs in ids:
        if refs == "":
            #Надо бы добавлять имя пресета, а не снапа
            pass
        else:
            ref_path = "./refs/" + str(refs) + ".npy"
            ref = np.load(ref_path)
            result = shifting_calculation(image,ref, crop_factor=crop)
            
            result = result + (refs,)
            # result = (result refs)
            res.append(result)
    # print(res)
    return res

def choise (reslist, thx, thy):
    sh = reslist.copy()
    drop_list = []
    #Выкалываю некорректные результаты
    for i in range(len(sh)):
        #Выкалываю результаты со всеми нулями к ак некорректные (хотя они могут быть и корректными)
        if (sh[i][0] == 0) and (sh[i][1] == 0) and (sh[i][0] == 0):
        # if (sh[i][0] == 0):
            drop_list.append(i)
        #Выкалываю значения со сдвигом больше трети кадра
        if sh[i][0]>thx:
            drop_list.append(i)
        elif sh[i][1]>thy:
            drop_list.append(i)
    #Удаляю эти элементы
    drop_list = sorted(drop_list , reverse=True)
    for dr in drop_list:
        sh.pop(dr)
    err_list = []
    for sherr in sh:
        err_list.append(sherr[2])
    if len(sh):
        res = sh[np.argmin (err_list)]
    else:
        res = [0,0,1000,""]
    return(res)

def optcorrection (fmm, camconf):
    ftype = str(camconf["model"]["focal"]["function"])
    if ftype == "xn": #x в степени
        kx = camconf["model"]["focal"]["funckx"]
        ky = camconf["model"]["focal"]["funcky"]
        if (len(kx) == 3) and  (len(ky) == 3):
            ax, bx, cx = kx
            ay, by, cy = ky
            resx = ax + bx* fmm**cx
            resy = ay + by* fmm**cy
            return resx, resy
        else:
            return fmm, fmm
    if ftype == "dexp": #Если сумма двух экспонент
        kx = camconf["model"]["focal"]["funckx"]
        ky = camconf["model"]["focal"]["funcky"]
        if (len(kx) == 5) and (len(ky) == 5):
            ax, bx, cx, dx, ex = kx
            resx = (ax + bx*np.exp(cx*fmm) + dx*np.exp(ex*fmm))
            ay, by, cy, dy, ey = ky
            resy = (ay + by*np.exp(cy*fmm) + dy*np.exp(ey*fmm))
            return resx, resy
        else:
            return fmm, fmm
    if ftype == "exp":
        kx = camconf["model"]["focal"]["funckx"]
        ky = camconf["model"]["focal"]["funcky"]
        if (len(kx) == 3) and (len(ky) == 5):
            ax, bx, cx = kx
            ay, by, cy = ky
            resx = ax + bx*np.exp(cx*fmm)
            resx = resx * float(camconf["fmx"])
            resy = ay + by*np.exp(cy*fmm)
            resy = resy * float(camconf["fmx"])
            return resx, resy
        else:
            return fmm, fmm
    else:
        return fmm, fmm
    
def image_translation (image, x, y):
    # print(x,y)
    tk = SimilarityTransform(scale=1, translation=(x, y))
    image = warp(image, tk, mode =  "edge")
    return image